@@ -34,13 +34,19 @@ Host: {}
 Warnings:
 """
 
+DEBUG_TEMPLATE = """\
+Task: Debug
+Host: {}
+{}
+"""
+
 
 def indent(text):
     '''Indent the given text by four spaces.'''
     return ''.join('    {}\n'.format(line) for line in text.splitlines())
 
 
-def print_failure_message(host_name, task_name, results):
+def print_failure_message(host_name, task_name, results, abridged_result):
     '''Print a human-readable error info from Ansible result dictionary.'''
     def is_script(results):
         return ('rc' in results and
@@ -79,7 +85,7 @@ def print_failure_message(host_name, task_name, results):
         print(indent(stderr))
     if display_full_results:
         print("Could not get an error message. Here is the Ansible output:")
-        pprint.pprint(results, indent=4)
+        pprint.pprint(abridged_result, indent=4)
     warnings = results.get('warnings', [])
     if warnings:
         print("Warnings:")
@@ -107,6 +113,7 @@ class CallbackModule(CallbackBase):
         host_name = result._host
         task_name = result._task.get_name()
         results = result._result  # A dict of the module name etc.
+        self._dump_results(results)
         warnings = results.get('warnings', [])
         # Print only tasks that produced some warnings:
         if warnings:
@@ -114,11 +121,35 @@ class CallbackModule(CallbackBase):
             for warning in warnings:
                 print("*", warning)
 
+        # Print the result of debug module
+        if (('invocation' in results) and
+           ('module_name' in results['invocation'])):
+
+            if ((results['invocation']['module_name'] == 'debug') and
+               ('module_args' in results['invocation'])):
+
+                output = ""
+
+                # Variable and its value
+                if 'var' in results['invocation']['module_args']:
+                    variable = results['invocation']['module_args']['var']
+                    value = results[variable]
+                    output = "{}: {}".format(variable, str(value))
+
+                # Debug message
+                elif 'msg' in results['invocation']['module_args']:
+                    output = "Message: {}".format(
+                             results['invocation']['module_args']['msg'])
+
+                print(DEBUG_TEMPLATE.format(host_name, output))
+
     def v2_runner_on_failed(self, result, **kwargs):
         host_name = result._host
         task_name = result._task.get_name()
 
         result_dict = result._result  # A dict of the module name etc.
+        abridged_result = self._dump_results(result_dict)
+
         if 'results' in result_dict:
             # The task is a list of items under `results`
             for item in result_dict['results']:
@@ -126,7 +157,8 @@ class CallbackModule(CallbackBase):
                     print_failure_message(host_name, task_name, item)
         else:
             # The task is a "normal" module invocation
-            print_failure_message(host_name, task_name, result_dict)
+            print_failure_message(host_name, task_name, result_dict,
+                                  abridged_result)
 
     def v2_runner_on_skipped(self, result, **kwargs):
         pass  # No need to print skipped tasks
